@@ -1,4 +1,3 @@
-# ex::lib
 #
 # Copyright (c) 200[789] Mons Anderson <mons@cpan.org>. All rights reserved
 # This program is free software; you can redistribute it and/or
@@ -11,7 +10,7 @@ ex::lib - The same as C<lib>, but makes relative path absolute.
 
 =head1 VERSION
 
-Version 0.03
+Version 0.04
 
 =head1 SYNOPSIS
 
@@ -60,27 +59,21 @@ under the same terms as Perl itself.
 
 =head1 AUTHOR
 
-Mons Anderson, C<< <mons@cpan.org> >>
+Mons Anderson, <mons@cpan.org>
 
 =cut
 
 use strict;
 use lib ();
-$ex::lib::VERSION = 0.03;
+use Cwd 3.29 qw(abs_path);
+$ex::lib::VERSION = 0.04;
+# TODO: check, fix, extend patterns
+$ex::lib::sep = {
+	( map { $_ => qr{[^\\/]+$}o } qw(mswin32 netware symbian dos) ),
+	( map { $_ => qr{[^:]+:?$}o } qw(macos) ),
+}->{lc$^O} || qr{[^/]+$}o;
 
-BEGIN {
-	# use constants is heavy :(
-	sub DEBUG () { 0 };
-	
-	# Load Cwd, if exists.
-	# There may be an XS version
-	eval { require Cwd; };
-	if ($@) {
-		*abs_path = \&_perl_abs_path;
-	}else{
-		Cwd->import('abs_path');
-	}
-}
+sub DEBUG () { 0 }; # use constants is heavy
 
 sub _carp  { require Carp; Carp::carp(@_)  }
 sub _croak { require Carp; Carp::croak(@_) }
@@ -91,11 +84,9 @@ sub mkapath($) {
 	# Prepare absolute base bath
 	my ($pkg,$file) = (caller($depth))[0,1];
 	warn "file = $file " if DEBUG > 1;
-	$file =~ s{[^/]+$}{}s;
+	$file =~ s/${ex::lib::sep}//s;
 	$file = '.' unless length $file;
 	warn "base path = $file" if DEBUG > 1;
-	#( my $p = $pkg  ) =~ s{::}{/}g;
-	#( my $f = $file ) =~ s/\Q$p.pm\E$//i;
 	my $f = abs_path($file) . '/';
 	warn "source dir = $f " if DEBUG > 1;
 	$f;
@@ -120,9 +111,7 @@ sub transform {
 sub import {
 	shift;
 	local $@; # Don't poison $@
-	
 	_croak("Bad usage. use ".__PACKAGE__." PATH") unless @_;
-
 	@_ = ( lib => transform @_ = @_ );
 	warn "use @_\n" if DEBUG > 0;
 	goto &lib::import;
@@ -139,77 +128,4 @@ sub unimport {
 	return;
 }
 
-
-# From Cwd;
-
-sub _perl_abs_path {
-	warn "Using perlish abs_path\n" if DEBUG;
-	my $start = @_ ? shift : '.';
-	my($dotdots, $cwd, @pst, @cst, $dir, @tst);
-
-	unless (@cst = stat( $start ))
-	{
-		_croak("$start: $!");
-		return '';
-	}
-
-	unless (-d _) {
-		# Make sure we can be invoked on plain files, not just directories.
-		# NOTE that this routine assumes that '/' is the only directory separator.
-	
-		my ($dir, $file) = $start =~ m{^(.*)/(.+)$}
-		or return cwd() . '/' . $start;
-	
-		# Can't use "-l _" here, because the previous stat was a stat(), not an lstat().
-		if (-l $start) {
-			my $link_target = readlink($start);
-			die "Can't resolve link $start: $!" unless defined $link_target;
-			
-			require File::Spec;
-				$link_target = $dir . '/' . $link_target
-					unless File::Spec->file_name_is_absolute($link_target);
-			
-			return abs_path($link_target);
-		}
-		
-		return $dir ? abs_path($dir) . "/$file" : "/$file";
-	}
-
-	$cwd = '';
-	$dotdots = $start;
-	do {
-		$dotdots .= '/..';
-		@pst = @cst;
-		local *PARENT;
-		unless (opendir(PARENT, $dotdots)) {
-			_carp("opendir($dotdots): $!");
-			return '';
-		}
-		unless (@cst = stat($dotdots)) {
-			_carp("stat($dotdots): $!");
-			closedir(PARENT);
-			return '';
-		}
-		if ($pst[0] == $cst[0] && $pst[1] == $cst[1]) {
-			$dir = undef;
-		}
-		else {
-			do {
-				unless (defined ($dir = readdir(PARENT))) {
-					_carp("readdir($dotdots): $!");
-					closedir(PARENT);
-					return '';
-				}
-				$tst[0] = $pst[0]+1 unless (@tst = lstat("$dotdots/$dir"))
-			}
-			while ($dir eq '.' || $dir eq '..' || $tst[0] != $pst[0] ||
-			$tst[1] != $pst[1]);
-		}
-		$cwd = (defined $dir ? "$dir" : "" ) . "/$cwd" ;
-		closedir(PARENT);
-	} while (defined $dir);
-	chop($cwd) unless $cwd eq '/'; # drop the trailing /
-	$cwd;
-}
-
 1;
